﻿/*
	Script:		main.js
	Gadget:		Net Meter
	Author:		Ben Coleman
	Version:	1.0.3
	Date:		11 June 2007
	Notes:		Tested on Google Desktop v5.1
	Changes:
			1.0.3	-	Changes displayed unit to Mbs if rate > 1000
*/

// default theme colour
options.putDefaultValue("milktheme_colour", COLORS[0]);

// global objects for getting WMI data
var refresher;
var collection;

// global variables some of which "mirror" values held in the options dictionary
var width = 135;
var down_val = 0;
var up_val = 0;
var total_down = 0;
var total_up = 0;
var max_down = 0;
var max_up = 0;
var tok;

// hook call so out custom menu function is called 
pluginHelper.onAddCustomMenuItems = addCustomMenuItems

//
// called when gadget is opened - init the gadget
//
function view_onOpen()
{
	// set colour theme
	Face.background = options("milktheme_colour");
	
	// reset bars to zero
	downbar.width = 0;	
	upbar.width = 0;

	// system defaults
	options.putDefaultValue("refresh_rate", 1);
	options.putDefaultValue("anim", true);
	options.putDefaultValue("auto_max", true);
	options.putDefaultValue("max_down", 100);
	options.putDefaultValue("max_up", 25);
	options.putDefaultValue("total_down", 0);
	options.putDefaultValue("total_up", 0);

	// Wbem WMI scripting code - creates a Swbemrefresher to get data from the 
	// Win32_PerfFormattedData_Tcpip_NetworkInterface class 
	// - NOTE. This class only exists on Windows XP and after 
	try {
		refresher = new ActiveXObject("WbemScripting.Swbemrefresher");
		var locator = new ActiveXObject("WbemScripting.SWbemLocator");
		var wmi_service = locator.ConnectServer("localhost", "root/CIMV2");
		wmi_service.Security_.ImpersonationLevel = 3;

		collection = refresher.AddEnum(wmi_service, "Win32_PerfFormattedData_Tcpip_NetworkInterface").objectSet;
	} catch(e) {
		alert("Failed to obtain WMI locator for\nWin32_PerfFormattedData_Tcpip_NetworkInterface class\n\nThis gadget only supports Windows XP or higher");
		event.returnValue = false;
	}
	
	// load totals from stored values
	total_down = options.getValue("total_down");
	total_up = options.getValue("total_up");

	// load max values from stored values
	max_down = options.getValue("max_down");
	max_up = options.getValue("max_up");

	// initialise the refresh function on a timer - to run every 'refresh_rate' seconds
	tok = SetInterval("refresh()", options.getValue("refresh_rate")*1000);
}

//
// main data refresh function - called every X seconds
//
function refresh()
{
	// refresh the WMI data, and load into a collection
	refresher.Refresh();
	col_enum = new Enumerator(collection);

	max_down = options.getValue("max_down");
	max_up = options.getValue("max_up");

	down_tot = 0;
	up_tot = 0;
	// loop through WMI results collection and get the bytes totals for each NIC adpater
	for(;!col_enum.atEnd();col_enum.moveNext()) {
		var i = col_enum.item();
		if(i.Name != "MS TCP Loopback interface") {
			down_tot += i.BytesReceivedPerSec;
			up_tot += i.BytesSentPerSec;
		}
	}

	// manipulate the upload & download data
	down = down_tot/1000	// convert to kbytes
	down_p = Math.min(down/max_down, 1.0);
	down_p_lab.innerText = "D: " + Math.round(down_p*100) + "%";
	// display the rate on the label (either as Kbs or Mbs)
	if(down < 1000) {
		down_k_lab.innerText = Math.round((down)*10)/10 + " Kbs";
	} else {
		down = down / 1000; // convert to mbytes
		down_k_lab.innerText = Math.round((down)*10)/10 + " Mbs";
	}

	up = up_tot/1000	// convert to kbytes
	up_p = Math.min(up/max_up, 1.0);
	up_p_lab.innerText = "U: " + Math.round(up_p*100) + "%";
	// display the rate on the label (either as Kbs or Mbs)
	if(up < 1000) {
		up_k_lab.innerText = Math.round((up)*10)/10+" Kbs";
	} else {
		up = up / 1000; // convert to mbytes
		up_k_lab.innerText = Math.round((up)*10)/10 + " Mbs";
	}

	// animate the percentage bars
	if(options.getValue("anim")) {
		// animation speed is 100 millis less than refresh rate
		rr = (options.getValue("refresh_rate") * 1000) - 100;
		beginAnimation("animateDownBar()", down_val, down_p*width, rr);
		beginAnimation("animateUpBar()", up_val, up_p*width, rr);
	} else {
		downbar.width = down_p*width;
		upbar.width = up_p*width;
	}

	// store totals
	total_down += down;
	total_up += up;
	options.putValue("total_down", total_down);
	options.putValue("total_up", total_up);
	if(total_down > 1000000) {
		down_str = Math.round((total_down/1000000)*100)/100 + "G";
	} else if(total_down > 1000) {
		down_str = Math.round((total_down/1000)*100)/100 + "M";
	} else {
		down_str = Math.floor(total_down)+"K";
	}
	if(total_up > 1000000) {
		up_str = Math.round((total_up/1000000)*100)/100 + "G";
	} else if(total_up > 1000) {
		up_str = Math.round((total_up/1000)*100)/100 + "M";
	} else {
		up_str = Math.floor(total_up)+"K";
	}	
	// display total on widget
	total_lab.innerText = "Total: " + down_str + " / " + up_str;

	// auto adjust the max values 
	if(options.getValue("auto_max")){
		if(down > max_down) {
			max_down = Math.floor(down);
			options.putValue("max_down", max_down);
		}
		if(up > max_up) {
			max_up = Math.floor(up);
			options.putValue("max_up", max_up);
		}
	}

	// store the 'old' percentage for use with the animation
	down_val = down_p*width;
	up_val = up_p*width;
}

//
// animation callback - download bar
//
function animateDownBar()
{
	downbar.width = event.value;
}

//
// animation callback - upload bar
//
function animateUpBar()
{
	upbar.width = event.value;
}

//
// called when user selected "Reset Totals" from the menu
//
function menu_resetTotals()
{
	options.putValue("total_up", 0);
	options.putValue("total_down", 0);
	total_down = 0;
	total_up = 0;
}

//
// called when any option is changed, refresh rate is the important one; we clear the timer a set a new one
//
function view_onOptionChanged()
{
	switch(event.propertyName)
	{
		case MILKTHEME_COLOR:
			Face.background = options( MILKTHEME_COLOR );
			break;
		case "refresh_rate":
			clearInterval(tok);
			tok = SetInterval("refresh()", options.getValue("refresh_rate")*1000);
			break;
	}
}


//
// add menu items to the main gadget menu
//
function view_onClose()
{
	// Empty for now...
}

//
// add menu items to the main gadget menu
//
function addCustomMenuItems(menu)
{
	menu.AddItem("Reset Totals", 0, menu_resetTotals);
}
